import subprocess
from dataclasses import dataclass
from typing import Optional
from pathlib import Path


class TransmissionCommand:
    def __init__(self):
        self.args = ["transmission-remote"]

    def add(self, path: Path) -> "TransmissionCommand":
        self.args.append("--add")
        self.args.append(str(path))
        return self

    def download_dir(self, path: Path) -> "TransmissionCommand":
        self.args.append("--download-dir")
        self.args.append(str(path))
        return self

    def auth(self, username: str, password: str) -> "TransmissionCommand":
        self.args.append("--auth")
        self.args.append(username + ":" + password)
        return self

    def list(self) -> "TransmissionCommand":
        self.args.append("-l")
        return self

    def exec(self) -> str:
        cp = subprocess.run(self.args, capture_output=True, check=True)
        return cp.stdout


@dataclass
class TransmissionRow:
    id_: Optional[str] = None
    done: Optional[str] = None
    have: Optional[str] = None
    eta: Optional[str] = None
    up: Optional[str] = None
    down: Optional[str] = None
    ratio: Optional[str] = None
    status: Optional[str] = None
    name: Optional[str] = None



import re
from pathlib import Path
from difflib import get_close_matches
from typing import Optional, Union


class TorrentData:
    def __init__(self, path: Path, **kwargs) -> None:
        if self._is_torrent_file(path):
            self.path: Path = path
        else:
            raise ValueError(f"Not a torrent file: {path}")
        self.data: Optional[list[Path]] = kwargs.get("data", None)
        self.tracker: Optional[str] = self.guess_tracker(path, **kwargs)

    def __str__(self) -> str:
        obj_vals = {
            "path": str(self.path),
        }
        if self.data:
            if isinstance(self.find_data, Path):
                obj_vals["data"] = str(self.data)
            else:
                obj_vals["data"] = [str(s) for s in self.data]
        if self.tracker:
            obj_vals["tracker"] = self.tracker
        return (
            f"TorrentFile(" + ", ".join([f"{k}={v}" for k, v in obj_vals.items()]) + ")"
        )

    @staticmethod
    def _is_torrent_file(path: Path) -> bool:
        if not path.is_file() or not path.suffix == ".torrent":
            return False
        return True

    def find_data(
        self, files: list[Path], cutoff: float = 0.9
    ) -> Optional[Union[Path, list[Path]]]:
        stems_hash = {ff.stem: ff for ff in files}
        close_matches = get_close_matches(
            word=self.path.stem, possibilities=stems_hash.keys(), cutoff=cutoff
        )
        if close_matches:
            close_matches = [stems_hash[s] for s in close_matches]
        self.data = close_matches

    @staticmethod
    def guess_tracker(path, read_len: int = 100) -> Optional[str]:
        with open(path, "rb") as F:
            line = F.readline()
            try:
                line_decoded = line[:read_len].decode("utf-8")
            except UnicodeDecodeError:
                return None
            tracker = re.match(
                r"^[\w]*:announce[0-9]*:http:\/\/([\w.-]+)", line_decoded
            )
        if tracker:
            return tracker.group(1)
        return None



import re
import time
from getpass import getpass
from argparse import ArgumentParser
from pathlib import Path
from typing import Optional

from .torrent import TorrentData
from .transmission import TransmissionCommand
from .methods import find_files, parse_list_output


def main(
    torrent_folder: Path,
    data_folder: Path,
    exclude_tracker: Optional[str],
    username: Optional[str],
    password: Optional[str],
    sleep: int,
):
    # Get the files
    torrent_files = [
        TorrentData(f) for f in find_files(torrent_folder, valid_exts=[".torrent"])
    ]
    data_files = find_files(data_folder, exclude_exts=[".torrent"])

    # Get the output from the client
    cmd = TransmissionCommand()
    if username and password:
        cmd.auth(username, password)
    cmd.list()
    res = cmd.exec()
    client_output = parse_list_output(res)

    for tf in torrent_files:
        if exclude_tracker and (
            tf.tracker is None or re.match(exclude_tracker, tf.tracker)
        ):
            continue
        tf.find_data(data_files)
        if tf.data:
            print(f"Torrent file @ {tf.path}\n  from tracker {tf.tracker}")
            if len(tf.data) == 1:
                download_dir = tf.data[0].parent
                print(f"matched data @ {download_dir}\nAdding to transmission...")
                
                cmd = TransmissionCommand()
                if username and password:
                    cmd.auth(username, password)
                cmd.add(tf.path).download_dir(download_dir)
                #res = cmd.exec()
                print("Transmission response:", res)
                time.sleep(sleep)
            elif len(tf.data) > 1:
                uniq_parents = {f.parent for f in tf.data}
                if len(uniq_parents) == 1:
                    print(f"matched data @ {tf.data[0].parent}")
                else:
                    print(f"matched data @ (multiple locations, skipping)")
                    for idx, f in enumerate(uniq_parents):
                        print(f"             {idx}. {f}")
            print("\n    ============    \n")


if __name__ == "__main__":
    parser = ArgumentParser(prog="transmission autoloader")
    parser.add_argument("-t", "--torrent_folder", type=Path, required=True)
    parser.add_argument("-d", "--data_folder", type=Path, required=True)
    parser.add_argument("-e", "--exclude_tracker", type=str, default=None)
    parser.add_argument("-u", "--username", type=str, default=None)
    parser.add_argument("-s", "--sleep", type=int, default=5)
    args = vars(parser.parse_args())

    args["password"] = getpass() if args["username"] else None
    main(**args)
